from django.core.urlresolvers import reverse

class MenuItem(object):
    """
    A container object for all the information about a single menu item.

    :param label: The label to be displayed for this menu item. Required.

    :param name: The symbolic name used in class names, ids and selections.
        If not specified, it is generated from the :attr:`label`.

    :param subitems: The list of :class:`MenuItem` instances for nested
        menus in :func:`bob.templatetags.bob.sidebar_menu` .

    :param href: The URL to which this menu item should link.
    :param view_name: The name of the URL rule to which this item should link.
    :param view_args: The positional arguments for the URL rule.
    :param view_kwargs: The keyword arguments for the URL rule.
    :param icon: A Bootstrap icon to show for this item.
    :param fugue_icon: A Fugue icon to show for this item, if available.
    :param collapsible: Whether the submenu should be collapsible. Default ``False``.
    :param collapsed: Whether the submenu should start collapsed. Default ``False``.
    :param indent: A string by which the item should be indented.
    """

    item_kind = 'link'

    def __init__(self, label, name=None, subitems=None, **kwargs):
        """
        """
        self.label = label
        if name is None:
            self.name = label.lower()
        else:
            self.name = name
        self.subitems = subitems
        self.kwargs = kwargs

    def get_href(self):
        href = self.kwargs.get('href')
        if href:
            return href
        view_name = self.kwargs.get('view_name')
        view_args = self.kwargs.get('view_args', [])
        view_kwargs = self.kwargs.get('view_kwargs', {})
        if view_name:
            return reverse(view_name, args=view_args, kwargs=view_kwargs)
        return ''


class Separator(object):
    """Show a separator in menus."""

    item_kind = 'separator'


class MenuHeader(object):
    """
    Show a header in menus.

    :param label: The text to display in the header.
    """

    item_kind = 'header'

    def __init__(self, label):
        self.label = label




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Unicode reader and writer for CSV.
Code from http://docs.python.org/library/csv.html
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import codecs
import cStringIO
import csv

from django.http import HttpResponse

class excel_semicolon(csv.excel):
    delimiter = b';'


class UTF8Recoder:
    """
    Iterator that reads an encoded stream and reencodes the input to UTF-8
    """
    def __init__(self, f, encoding):
        self.reader = codecs.getreader(encoding)(f)

    def __iter__(self):
        return self

    def next(self):
        return self.reader.next().encode("utf-8")

class UnicodeReader:
    """
    A CSV reader which will iterate over lines in the CSV file "f",
    which is encoded in the given encoding.
    """

    def __init__(self, f, dialect=excel_semicolon, encoding="utf-8", **kwds):
        f = UTF8Recoder(f, encoding)
        self.reader = csv.reader(f, dialect=dialect, **kwds)

    def next(self):
        row = self.reader.next()
        return [unicode(s, "utf-8") for s in row]

    def __iter__(self):
        return self


class UnicodeWriter:
    """
    A CSV writer which will write rows to CSV file "f",
    which is encoded in the given encoding.
    """

    def __init__(self, f, dialect=excel_semicolon, encoding="utf-8", **kwds):
        # Redirect output to a queue
        self.queue = cStringIO.StringIO()
        self.writer = csv.writer(self.queue, dialect=dialect, **kwds)
        self.stream = f
        self.encoder = codecs.getincrementalencoder(encoding)()

    def writerow(self, row):
        self.writer.writerow([s.encode("utf-8") for s in row])
        # Fetch UTF-8 output from the queue ...
        data = self.queue.getvalue()
        data = data.decode("utf-8")
        # ... and reencode it into the target encoding
        data = self.encoder.encode(data)
        # write to the target stream
        self.stream.write(data)
        # empty queue
        self.queue.truncate(0)

    def writerows(self, rows):
        for row in rows:
            self.writerow(row)


def make_csv_response(data=[], filename='export.csv'):
    """
    Create a HTTP response for downloading a CSV file with provided data.

    :param data - list of rows of data
    :param filename - the name of the file to be downloaded

    How to use:
    >>> rows = [
    ...    ['CAR', 'COLOR'],
    ...    ['Ford', 'Red'],
    ...    ['BMW', 'Black'],
    ...]
    >>> response = make_csv_response(data=rows, filename='myfile.csv')
    """

    f = cStringIO.StringIO()
    UnicodeWriter(f).writerows((unicode(item) for item in row) for row in data)
    response = HttpResponse(f.getvalue(), content_type='application/csv')
    disposition = 'attachment; filename=%s' % filename
    response['Content-Disposition'] = disposition
    return response



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import cStringIO as StringIO

from django.core.paginator import Paginator, EmptyPage
from django.db.models import FieldDoesNotExist
from django.http import HttpResponse

from bob import csvutil


class DataTableColumn(object):
    """
    A container object for all the information about a columns header

    :param header_name - name to display in columns header
    :param field - field name in model
    :param type - icon type generated by bob header_table tag
    :param selectable - header in bob tag is selectable, if you use this option
     you must include bootstrap/js/bob.js file in your template
    :param bob_tag - set if the column is to be generated by bob tag
    :param sort_expression - example `book__size` (book.size)
    :param export - set when the column is to be exported
    """
    def __init__(self, header_name, field=None, type=None, selectable=None,
                 bob_tag=None, sort_expression=None, export=None):
        self.header_name = header_name
        self.field = field
        self.type = type
        self.selectable = selectable
        self.bob_tag = bob_tag
        self.sort_expression = sort_expression
        self.export = export


class DataTableMixin(object):
    """Add this Mixin to your django view to handle page pagination.

    In your controller:
    1. Inherit from this mixin
    2. Define ``rows_per_page`` attribute
    3. Define ``csv_file_name`` attribute
    4. Define ``sort_variable_name`` attribute
    5. Define ``export_variable_name`` attribute
    4. Define columns attribute containing a list of
    :py:class:bob.data_table.DataTableColumn objects
    5. In get() function call
    :py:meth:bob.data_table.DataTableMixin.data_table_query(your_query, columns).

    Result is stored in the
    :py:attr:bob.data_table.DataTableMixin.page_contents
    Data for template can be obtained from
    :py:method:bob.data_table.DataTableMixin.get_context_data_paginator()
    dict to the template.

    In your template add code::

    {% pagination bob_page url_query=url_query show_all=0 show_csv=0 fugue_icons=1 sort_variable_name %}

    where ``query_variable_name`` - is the name of the attribute used for pagination, and::

    {% table_header columns url_query sort fugue_icons%}

    All done!

    """

    csv_file_name = 'file.csv'
    query_variable_name = 'page'
    export_variable_name = 'export'
    rows_per_page = 15
    sort = None

    def get_csv_header(self):
        """Generate a list of columns used in csv file header"""
        return [col.header_name for col in self.columns if col.export]

    def get_cell(self, obj, field, model):
        """Returns the contents of a cell
        :param obj: model instance
        :param field: object field name
        :param model: object model instance
        :return: contents of the cell
        """
        cell = ''
        if obj:
            try:
                model._meta.get_field_by_name(field)[0].choices
                cell = getattr(obj, 'get_' + field + '_display')()
            except (FieldDoesNotExist, AttributeError):
                pass
            if not cell:
                try:
                    cell = getattr(obj, field)
                except AttributeError:
                    pass
        return cell

    def get_context_data_paginator(self, **kwargs):
        """Returns paginator data dict, crafted for usage in template."""
        return {
            'bob_page': self.page_contents,
        }

    def data_table_query(self, queryset):
        queryset = self.sort_queryset(queryset)
        if self.export_requested():
            self.response = self.do_csv_export(queryset)
        else:
            self.page_contents = self._paginate(queryset)

    def sort_queryset(self, queryset, sort=None):
        """Sorted queryset based on sort param"""
        self.prepare_sortable_columns()
        if sort is None:
            sort = self.request.GET.get(self.sort_variable_name)
        if sort:
            sort_columns = self.sortable_columns.get(sort.strip('-'))
            if sort_columns:
                if sort.startswith('-'):
                    sort_columns = '-' + sort_columns
                queryset = queryset.order_by(sort_columns)
                self.sort = sort_columns
        return queryset

    def prepare_sortable_columns(self):
        self.sortable_columns = dict(
            (c.field, c.sort_expression) for c in self.columns
            if c.sort_expression
        )

    def _paginate(self, queryset):
        """Internal pagination function"""
        page = self.request.GET.get(self.query_variable_name) or 1
        try:
            self.page_number = int(page)
        except ValueError:
            self.page_number = 1
        self.paginator = Paginator(queryset, self.rows_per_page)
        try:
            page_contents = self.paginator.page(self.page_number)
        except EmptyPage:
            page_contents = self.paginator.page(1)
        return page_contents

    def export_requested(self, *args, **kwargs):
        """Returns True if csv export was requested by the user or
        False in other case
        """
        export = self.request.GET.get(self.export_variable_name)
        return export == 'csv'

    def get_csv_data(self, queryset):
        """Should returns generic rows.
        Override this method in inherited class
        """

    def make_csv_response(self, data):
        return csvutil.make_csv_response(
            data=data, filename=self.csv_file_name)

    def do_csv_export(self, queryset):
        return self.make_csv_response(self.get_csv_data(queryset))



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from django import forms
from django.utils.safestring import mark_safe
from django.forms.util import flatatt
from django.utils.html import escape
from django.utils import simplejson as json

class AutocompleteWidget(forms.Select):
    """A choice widget using a text field with Bootstrap's autocomplete."""

    def render(self, name, value, attrs=None, choices=()):
        if value is None: value = ''
        final_attrs = self.build_attrs(attrs, name=name)
        labels = dict(self.choices)
        output = [
            u'<input type="text" autocomplete="off" data-provide="typeahead" data-items="10" data-source="%s" value="%s" %s>' % (
                escape(
                    json.dumps(zip(*self.choices)[1] if self.choices else [])
                ),
                escape(labels.get(value, value)),
                flatatt(final_attrs))
        ]
        return mark_safe('\n'.join(output))

    def value_from_datadict(self, data, files, name):
        value = data.get(name, None)
        if value == '---------':
            return None
        values = dict((label, value) for value, label in self.choices )
        return values.get(value, value)


class DateTimeWidget(forms.DateInput):
    """A datetime widget using the HTML5 date picker."""

    input_type = 'datetime'


class DateWidget(forms.DateInput):
    """A date widget using jQuery date picker."""

    def render(self, name, value='', attrs=None, choices=()):
        attr_class =  escape(self.attrs.get('class', ''))
        attr_placeholder = escape(self.attrs.get('placeholder', ''))
        output = ('<input type="text" name="%s" class="datepicker %s" '
                  'placeholder="%s" value="%s" data-date-format="yyyy-mm-dd">')
        return mark_safe(output % (escape(name), attr_class,
                                   attr_placeholder, escape(value or '')))



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import json
import datetime

from django import template
from django.utils.safestring import mark_safe
from django.utils.html import conditional_escape as esc
from django.utils.timesince import timesince
from bob.forms.dependency import DependencyForm


register = template.Library()


@register.simple_tag
def bob_icon(name, is_white=False):
    """
    Display a bootstrap icon.

    :param name: The name of the icon to display.
    :param is_white: Whether the icon should be white (for dark background).
    """

    white = ' icon-white' if is_white else ''
    return mark_safe('<i class="icon-%s%s"></i>' % esc(name, white))


@register.inclusion_tag('bob/main_menu.html')
def main_menu(items, selected, title=None, search=None, white=False,
              position='', title_url="/"):
    """
    Show main menu bar.

    :param items: The list of :class:`bob.menu.MenuItem` instances to show.
    :param selected: The :data:`name` of the currently selected item.
    :param title: The title to show in the menu bar.
    :param search: The URL for the search form.
    :param white: If True, the menu bar will be white.
    :param position: Empty, or one of ``'fixed'``, ``'static'``, ``'bottom'``.
    """

    return {
        'items': items,
        'selected': selected,
        'title': title,
        'search': search,
        'position': position,
        'white': bool(white),
        'title_url': title_url,
        }


@register.inclusion_tag('bob/tab_menu.html')
def tab_menu(items, selected, side=None):
    """
    Show a menu in form of tabs.

    :param items: The list of :class:`bob.menu.MenuItem` instances to show.
    :param selected: The :data:`name` of the currently selected item.
    :param side: The direction of tabs, may be on of ``"left"``, ``"right"``,
        ``"top"`` or ``"bottom"``. Defaults to ``"top"``.
    """

    return {
        'items': items,
        'selected': selected,
        'side': side,
        }


@register.inclusion_tag('bob/sidebar_menu.html')
def sidebar_menu(items, selected):
    """
    Show menu in a sidebar.

    :param items: The list of :class:`bob.menu.MenuItem` instances to show.
    :param selected: The :data:`name` of the currently selected item.
    """

    return {
        'items': items,
        'selected': selected,
        }


@register.inclusion_tag('bob/sidebar_menu_subitems.html')
def sidebar_menu_subitems(item, selected):
    """
    Show subitems of a menu in a sidebar.
    """

    return {
        'item': item,
        'selected': selected,
        }


@register.inclusion_tag('bob/pagination.html')
def pagination(page, show_all=False, show_csv=False,
               fugue_icons=False, url_query=None, neighbors=1,
               query_variable_name='page', export_variable_name='export'):
    """
    Display pagination for a list of items.

    :param page: Django's paginator page to display.
    :param show_all: Whether to show a link for disabling pagination.
    :param show_csv: Whether to show a link to CSV download.
    :param fugue_icons: Whether to use Fugue icons or Bootstrap icons.
    :param url_query: The query parameters to add to all page links.
    :param neighbors: How many neighboring pages to show in paginator.
    """

    if not page:
        return {
            'show_all': show_all,
            'show_csv': show_csv,
            'fugue_icons': fugue_icons,
            'url_query': url_query,
            'export_variable_name': export_variable_name,
        }
    paginator = page.paginator
    page_no = page.number
    pages = paginator.page_range[max(0, page_no - 1 - neighbors):
    min(paginator.num_pages, page_no + neighbors)]

    if 1 not in pages:
        pages.insert(0, 1)
        pages.insert(1, '...')
    if paginator.num_pages not in pages:
        pages.append('...')
        pages.append(paginator.num_pages)
    urls = []
    for item in pages:
        if item == '...':
            urls.append(changed_url(url_query, query_variable_name, page_no))
        else:
            urls.append(changed_url(url_query, query_variable_name, item))
    url_pages = zip(pages, urls)
    return {
        'paginator': paginator,
        'page_no': page_no,
        'page': page,
        'pages': pages,
        'show_all': show_all,
        'show_csv': show_csv,
        'fugue_icons': fugue_icons,
        'url_query': url_query,
        'url_previous_page': changed_url(url_query, query_variable_name, page_no-1),
        'url_next_page': changed_url(url_query, query_variable_name, page_no+1),
        'url_pages': url_pages,
        'url_all': changed_url(url_query, query_variable_name, 0),
        'export_variable_name': export_variable_name,
    }


def changed_url(query, name, value):
    if not query:
        return '%s=%s' % (name, value)
    query = query.copy()
    if value is not None and value not in ('1', 1):
        query[name] = value
    else:
        try:
            del query[name]
        except KeyError:
            pass
    return query.urlencode()


@register.filter
def bob_export(query, export):
    """Modify the query string of an URL to change the ``export`` argument."""

    if not query:
        return 'export=%s' % export
    query = query.copy()
    if export:
        query['export'] = export
    else:
        try:
            del query['export']
        except KeyError:
            pass
    return query.urlencode()


@register.filter
def timesince_limited(d):
    """
    Display time between given date and now in a human-readable form if the
    time span is less than a day, otherwise display the date normally.

    :param d: The date to display.
    """
    today = datetime.datetime.now()
    delta = datetime.timedelta
    interval = today - d
    if today.strftime('%Y-%m-%d') == d.strftime('%Y-%m-%d'):
        if interval < delta(days=0, hours=1):
            return timesince(d) + ' ago '
        else:
            return d.strftime('%H:%M')
    else:
        return d


@register.inclusion_tag('bob/form.html')
def form(form, action="", method="POST", fugue_icons=False,
         css_class="form-horizontal", title="", submit_label='Save'):
    """
    Render a form.

    :param form: The form to render.
    :param action: The submit URL.
    :param method: The submit method, either ``"GET"`` or ``"POST"``.
    :param fugue_icons: Whether to use Fugue or Bootstrap icon.
    :param css_class: The CSS class to use for the ``<form>`` tag.
    :param title: Form title.
    :param submit_label: Submit button label.
    """
    return {
        'form': form,
        'action': action,
        'title': title,
        'method': method,
        'fugue_icons': fugue_icons,
        'css_class': css_class,
        'submit_label': submit_label,
    }


@register.inclusion_tag('bob/form.html')
def form_horizontal(*args, **kwargs):
    return form(*args, **kwargs)


@register.inclusion_tag('bob/table_header.html')
def table_header(columns=None, url_query=None, sort=None, fugue_icons=False,
                 sort_variable_name='sort'):
    """
    Render a table header with sorted column options

    :param columns: a list of objects of
                    type :py:class:bob.data_table.DataTableColumn
    :param url_query: The query parameters to add to all page links
    :param sort: means that the column is now sorted
    :param fugue_icons: Whether to use Fugue icons or Bootstrap icons.

    """
    return {
        'columns': columns,
        'sort': sort,
        'url_query': url_query,
        'fugue_icons': fugue_icons,
        'sort_variable_name': sort_variable_name,
    }

@register.simple_tag
def bob_sort_url(query, field, sort_variable_name, type):
    """Modify the query string of an URL to change the ``sort_variable_name``
    argument.
    """
    query = query.copy()
    if type == 'desc':
        query[sort_variable_name] = '-' + field
    elif type == 'asc':
        query[sort_variable_name] = field
    return query.urlencode()

@register.simple_tag
def bob_export_url(query, value, export_variable_name='export'):
    """Modify the query string of an URL to change the ``export_variable_name``
    argument.
    """
    if not query:
        return '%s=%s' % (export_variable_name, value)
    query = query.copy()
    if value:
        query[export_variable_name] = value
    else:
        try:
            del query[export_variable_name]
        except KeyError:
            pass
    return query.urlencode()

@register.simple_tag
def dependency_data(form):
    """Render the data-bob-dependencies tag if this is a DependencyForm"""

    if not isinstance(form, DependencyForm):
        return ''
    return 'data-bob-dependencies="{0}"'.format(
        esc(json.dumps(form.get_dependencies_for_js())))

@register.inclusion_tag('bob/field_wrapper.html')
def field_wrapper(field):
    """Render the full control-group tag of a field."""
    return {'field': field}



# -*- coding: utf-8 -*-
"""All the objects needed to use the dependency mechanism"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from collections import namedtuple, Container

from django.db.models import Model

SHOW = 'SHOW'

Dependency = namedtuple('Dependency', ('slave', 'master', 'value', 'action'))

class Dependency(object):
    """The single dependency: if master has value then do action on slave."""

    def __init__(self, slave, master, value, action):
        self.slave = slave
        self.master = master
        self.value = value
        self.action = action

    def met(self, data):
        """Return true if condition is met."""
        val = data.get(self.master)
        if val is None:
            return False
        if isinstance(self.value, Container):
            return val in self.value
        else:
            return val == self.value

class DependencyForm(object): # Can't inherit Form due to metaclass conflict
    """Form mixin with dependency feature."""

    dependencies = []

    def clean(self):
        data = super(DependencyForm, self).clean()
        for dep in self.dependencies:
            if not dep.met(data):
                if dep.action == SHOW:
                    self._errors.pop(dep.slave, None)
        return data
    
    def _format_single_val_for_js(self, val):
        """Return the appropriate js representation of a single value."""
        if isinstance(val, Model):
            return str(val.id)
        return val


    def get_dependencies_for_js(self):
        """Return the dependencies in a format ready to be JSON serialized for
        JavaScript"""
        result = []
        for dep in self.dependencies:
            if isinstance(dep.value, Container):
                value = [self._format_single_val_for_js(v) for v in dep.value]
            else:
                value = self._format_single_val_for_js(dep.value)
            result.append({
                'slave': dep.slave, 'master': dep.master,
                'value': value, 'action': dep.action})
        return result



from setuptools import setup, find_packages

setup(
    name='django-bob',
    description='A set of helper tags and templates for using bootstrap in django.',
    long_description="""
Bob is a library of templates, template tags, helper functions and form widgets
that make it easier to use the Twitter's Bootstrap framework with Django.
    """,
    version='1.5.10',
    license='Apache 2',
    url='http://bob.readthedocs.org/',
    author='Radomir Dopieralski',
    author_email='devel@sheep.art.pl',
    keywords='bootstrap django css html',
    packages=find_packages(),
    install_requires=['distribute', 'django'],
    platforms='any',
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Scientific/Engineering :: Human Machine Interfaces',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: User Interfaces',
    ],
)
